/**Class to simulate partially I/O bound process.
 */
public class Work implements Comparable
    {
    /**Average CPU utilisation of this job (1.0 means CPU bound, 0.0 IO-bound).
     */
    public static final float CPU_UTIL = 0.3f;

    /**Construct a work unit. */
    public Work(int i) { workLength = i; }

    /**Remember work length. */
    final int workLength;

    /**Do the work... */
    public void doWork()
        {
        System.out.println("  [Doing job " + hashCode() + " of length " + workLength + "...]");

        /**Get different but repeatable behaviour for each task length. */
        final int timeSlice = (workLength % 353) + 255;

        // Alternately busy-wait and sleep...
        for(int timeLeft = workLength; timeLeft > 0; timeLeft -= timeSlice)
            {
            final int thisSlice = Math.min(timeLeft, timeSlice);

            // Compute time to burn cycles and time to sleep.
            final int busyTime = (int) (CPU_UTIL * thisSlice);
            final int sleepTime = Math.max(1, thisSlice - busyTime);
//System.err.println("B|S " + busyTime + "|" + sleepTime);

            // Burn cycles.
            final long stopTime = System.currentTimeMillis() + busyTime;
            while(System.currentTimeMillis() < stopTime)
                {
                // Burn cycles like there is no tomorrow...
                }

            // Emulate being I/O bound...
            try { Thread.sleep(sleepTime); }
            catch(InterruptedException e) { }
            }

        System.out.println("  [Finished job " + hashCode() + " of length " + workLength + "...]");
        }

    /**We make this class Comparable so that it can be used in all collections.
     * We order by hashCode to keep all instances distinct.
     */
    public int compareTo(Object o)
        {
        final Work otherW = (Work) o;
        final int h = hashCode();
        final int oH = otherW.hashCode();
        if(h < oH) { return(-1); }
        if(h > oH) { return(1); }
        return(0);
        }
    }
